#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const fs_1 = require("fs");
const globby_1 = require("globby");
const path_1 = require("path");
program
    .version('0.0.1')
    .option('-p, --project <file>', 'path to tsconfig.json');
program.on('--help', () => {
    console.log(`
  $ tscpath -p tsconfig.json
`);
});
program.parse(process.argv);
const { project } = program;
if (!project) {
    throw new Error('--project must be specified');
}
const configFile = path_1.join(process.cwd(), project);
const { compilerOptions: { baseUrl, outDir, paths }, include, } = require(configFile);
const configDir = path_1.dirname(configFile);
const basePath = path_1.join(configDir, baseUrl);
const toAbsPath = (x) => path_1.join(basePath, x);
const outPath = toAbsPath(outDir);
const srcPaths = include
    .filter((x) => x.endsWith('/**/*'))
    .map((x) => toAbsPath(x.replace('/**/*', '')));
const srcRoot = srcPaths.length === 1 ? srcPaths[0] : basePath;
const outToSrc = (x) => path_1.join(srcRoot, path_1.relative(outPath, x));
const aliases = Object.keys(paths).map((alias) => ({
    prefix: alias.replace(/\*$/, ''),
    aliasPaths: paths[alias].map((p) => path_1.join(basePath, p.replace(/\*$/, ''))),
}));
const toRelative = (from, x) => {
    const rel = path_1.relative(from, x);
    return rel.startsWith('.') ? rel : `./${rel}`;
};
const relToConfig = (x) => toRelative(configDir, x);
console.log(`
tsconfig:
  base = ${relToConfig(basePath)}
  src = ${relToConfig(srcRoot)}
  sources = ${srcPaths.map((x) => relToConfig(x + '/**/*'))}
  out = ${relToConfig(outPath)}
  aliases = ${aliases.map(({ prefix }) => prefix).join(', ')}
`);
const exts = ['.js', '.jsx', '.ts', '.tsx', '.d.ts'];
const absToRel = (modulePath, outFile) => {
    const alen = aliases.length;
    for (let j = 0; j < alen; j += 1) {
        const { prefix, aliasPaths } = aliases[j];
        if (modulePath.startsWith(prefix)) {
            const modulePathRel = modulePath.substring(prefix.length);
            const src = outToSrc(outFile);
            const outRel = path_1.relative(basePath, outFile);
            console.log(`${outRel} (source: ${path_1.relative(basePath, src)}):`);
            console.log(`\timport '${modulePath}'`);
            const len = aliasPaths.length;
            for (let i = 0; i < len; i += 1) {
                const apath = aliasPaths[i];
                const moduleSrc = path_1.join(apath, modulePathRel);
                if (fs_1.existsSync(moduleSrc) ||
                    exts.some((ext) => fs_1.existsSync(moduleSrc + ext))) {
                    const rel = toRelative(path_1.dirname(src), moduleSrc);
                    console.log(`\treplacing '${modulePath}' -> '${rel}' referencing ${path_1.relative(basePath, moduleSrc)}`);
                    return rel;
                }
            }
            console.log(`\tcould not replace ${modulePath}`);
        }
    }
    return modulePath;
};
const requireRegex = /(?:import|require)\(['"]([^'"]*)['"]\)/g;
const importRegex = /from ['"]([^'"]*)['"]/g;
const replaceImportStatement = (orig, matched, outFile) => {
    const index = orig.indexOf(matched);
    return (orig.substring(0, index) +
        absToRel(matched, outFile) +
        orig.substring(index + matched.length));
};
const replaceAlias = (text, outFile) => text
    .replace(requireRegex, (orig, matched) => replaceImportStatement(orig, matched, outFile))
    .replace(importRegex, (orig, matched) => replaceImportStatement(orig, matched, outFile));
const files = globby_1.sync(`${outPath}/**/*.{js,jsx,ts,tsx}`, {
    dot: true,
    noDir: true,
}).map((x) => path_1.resolve(x));
const flen = files.length;
for (let i = 0; i < flen; i += 1) {
    const file = files[i];
    const text = fs_1.readFileSync(file, 'utf8');
    const newText = replaceAlias(text, file);
    if (text !== newText) {
        fs_1.writeFileSync(file, newText, 'utf8');
    }
}
//# sourceMappingURL=index.js.map